==> pybo-0.1.3.tar.gz: pybo-0.1.3/pybo/splitaffixed.py <==
from .token import Token


class SplitAffixed:
    """
    A class to split Token objects produced by BoTokenizer.Tokenizer
    if they contain affixed particles
    """

    def __init__(self):
        self.tag_sep = 'ᛃ'

    def split(self, tokens):
        """
        Splits in place the tokens containing affixed particles

        :param tokens: list of Token objects
        """
        def affixed_matcher(token):
            return self.tag_sep in token.tag and self.tag_sep * 3 not in token.tag

        t = 0
        while t <= len(tokens) - 1:
            if affixed_matcher(tokens[t]):
                # split token containing the affixed particle
                token1, token2 = self.split_token(tokens[t])

                # replace the original token with the two new ones
                tokens[t: t + 1] = [token1, token2]
                t += 1  # increment once more to account for the newly split token
            t += 1

    def split_token(self, token):
        def split_contents(content, split_idx):
            return content[:split_idx], content[split_idx:]

        def split_char_groups(char_groups, split_idx):
            token_groups, affix_groups = {}, {}
            for idx, group in sorted(char_groups.items()):
                if idx < split_idx:
                    token_groups[idx] = group
                else:
                    affix_groups[idx] = group
            return token_groups, affix_groups

        def split_lengths(length, split_idx):
            return split_index, length - split_idx

        def split_syls(syls, split_idx):
            t_syls, a_syls = [], []
            for syl in syls:
                if syl[-1] <= split_idx:
                    t_syls.append(syl)
                else:
                    token_part, affix_part = [], []
                    for i in syl:
                        if i < split_idx:
                            token_part.append(i)
                        else:
                            affix_part.append(i - split_idx)
                    t_syls.append(token_part)
                    a_syls.append(affix_part)
            return t_syls, a_syls

        pos, split_index, affix_type, aa = self.get_affix_info(token)
        token_content, affix_content = split_contents(token.content, split_index)
        token_char_groups, affix_char_groups = split_char_groups(token.char_groups, split_index)
        token_length, affix_length = split_lengths(token.length, split_index)
        token_tag, affix_tag = '{}{}{}'.format(pos, self.tag_sep, self.tag_sep), \
                               '{}{}{}{}{}'.format('PART', self.tag_sep, affix_type, self.tag_sep, aa)
        token_start, affix_start = token.start, token.start + token_length
        token_syls, affix_syls = split_syls(token.syls, split_index)

        # un-affixed token object
        t = Token()
        t.content = token_content
        t.chunk_type = token.chunk_type
        t.start = token_start
        t.length = token_length
        t.syls = token_syls
        t.tag = token_tag
        t.char_groups = token_char_groups

        # affix token object
        a = Token()
        a.content = affix_content
        a.chunk_type = token.chunk_type
        a.start = affix_start
        a.length = affix_length
        a.syls = affix_syls
        a.tag = affix_tag
        a.char_groups = affix_char_groups

        return t, a

    def get_affix_info(self, token):
        pos, affix_type, affix_len, aa = token.tag.split(self.tag_sep)
        start_index = len(token.content) - 1 - int(affix_len)
        return pos, start_index, affix_type, aa

==> pybo-0.1.6.tar.gz: pybo-0.1.6/pybo/splitaffixed.py <==
from .token import Token
from .helpers import AFFIX_SEP


class SplitAffixed:
    """
    A class to split Token objects produced by BoTokenizer.Tokenizer
    if they contain affixed particles

    .. note: should be rewritten with the inspiration of TokenSplit and
            TokenMerge. Could be a function instead of a class
    """

    def __init__(self):
        pass

    def split(self, tokens):
        """
        Splits in place the tokens containing affixed particles

        :param tokens: list of Token objects
        """
        def affixed_matcher(token):
            return AFFIX_SEP in token.tag and AFFIX_SEP * 3 not in token.tag

        t = 0
        while t <= len(tokens) - 1:
            if affixed_matcher(tokens[t]):
                # split token containing the affixed particle
                token1, token2 = self.split_token(tokens[t])

                # replace the original token with the two new ones
                tokens[t: t + 1] = [token1, token2]
                t += 1  # increment once more to account for the newly split token
            t += 1

    def split_token(self, token):
        def split_contents(content, split_idx):
            return content[:split_idx], content[split_idx:]

        def split_char_groups(char_groups, split_idx):
            token_groups, affix_groups = {}, {}
            for idx, group in sorted(char_groups.items()):
                if idx < split_idx:
                    token_groups[idx] = group
                else:
                    affix_groups[idx] = group
            return token_groups, affix_groups

        def split_lengths(length, split_idx):
            return split_index, length - split_idx

        def split_syls(syls, split_idx):
            t_syls, a_syls = [], []
            for syl in syls:
                if syl[-1] < split_idx:
                    t_syls.append(syl)
                else:
                    token_part, affix_part = [], []
                    for i in syl:
                        if i < split_idx:
                            token_part.append(i)
                        else:
                            affix_part.append(i - split_idx)
                    t_syls.append(token_part)
                    a_syls.append(affix_part)
            return t_syls, a_syls

        pos, split_index, affix_type, aa = self.get_affix_info(token)
        token_content, affix_content = split_contents(token.content, split_index)
        token_char_groups, affix_char_groups = split_char_groups(token.char_groups, split_index)
        token_length, affix_length = split_lengths(token.length, split_index)
        token_tag, affix_tag = '{}{}{}{}'.format(pos, AFFIX_SEP, AFFIX_SEP, AFFIX_SEP), \
                               '{}{}{}{}{}'.format('PART', AFFIX_SEP, affix_type, AFFIX_SEP, AFFIX_SEP, aa)
        token_start, affix_start = token.start, token.start + token_length
        token_syls, affix_syls = split_syls(token.syls, split_index)

        # un-affixed token object
        t = Token()
        t.content = token_content
        t.chunk_type = token.chunk_type
        t.start = token_start
        t.length = token_length
        t.syls = token_syls
        t.tag = token_tag
        t.char_groups = token_char_groups
        t.affix = False
        t.affixed = True
        if aa:
            t.aa_word = True
        t.get_pos_n_aa()

        # affix token object
        a = Token()
        a.content = affix_content
        a.chunk_type = token.chunk_type
        a.start = affix_start
        a.length = affix_length
        a.syls = affix_syls
        a.tag = affix_tag
        a.char_groups = affix_char_groups
        a.affix = True
        a.affixed = False
        a.get_pos_n_aa()

        return t, a

    def get_affix_info(self, token):
        pos, affix_type, affix_len, aa = token.tag.split(AFFIX_SEP)
        start_index = token.syls[-1][-int(affix_len)]
        return pos, start_index, affix_type, aa
